/*
 * Copyright (C) 2022 Jonathan Certes
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * To show that having a reset with no constraint can lead to never reaching the
 * maximum value.
 */
module up_counter(
  input clk,
  input reset,
  output[3:0] counter
);
  reg [3:0] counter_up;

  always @( posedge(clk) or posedge(reset) )
  begin
    if ( reset ) begin
      counter_up <= 4'd0;
    end
    else begin
      counter_up <= counter_up + 4'd1;
    end
  end

  assign counter = counter_up;
endmodule

/******************************************************************************/

/**
 * To show that, with no reset and a FAIRNESS constraint on clk, maximum is
 * always reached.
 */
module no_reset(
  input clk,
  output[3:0] counter
);
  reg [3:0] counter_up;

  always @( posedge(clk) )
  begin
    counter_up <= counter_up + 4'd1;
  end

  assign counter = counter_up;
endmodule

/******************************************************************************/

/**
 * To show that posedge operators are not handled by verilog2smv, having a
 * boolean that is always set to 1 produces the same effect.
 */
module no_clk(
  output[3:0] counter
);
  reg [3:0] counter_up;

  wire clk;
  assign clk = 1'b1;

  always @( posedge(clk) )
  begin
    counter_up <= counter_up + 4'd1;
  end

  assign counter = counter_up;
endmodule

